<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;
if ( ! class_exists( 'Order_Manager_System' ) ) :

	/**
	 * Main Order_Manager_System Class.
	 *
	 * @package		OMS
	 * @subpackage	Classes/Order_Manager_System
	 * @since		1.0.0
	 * @author		OMS Support
	 */
	final class Order_Manager_System {

		/**
		 * The real instance
		 *
		 * @access	private
		 * @since	1.0.0
		 * @var		object|Order_Manager_System
		 */
		private static $instance;

		/**
		 * OMS helpers object.
		 *
		 * @access	public
		 * @since	1.0.0
		 * @var		object|Order_Manager_System_Helpers
		 */
		public $helpers;

		/**
		 * OMS settings object.
		 *
		 * @access	public
		 * @since	1.0.0
		 * @var		object|Order_Manager_System_Settings
		 */
		public $settings;

		/**
		 * Throw error on object clone.
		 *
		 * Cloning instances of the class is forbidden.
		 *
		 * @access	public
		 * @since	1.0.0
		 * @return	void
		 */
		public function __clone() {
			_doing_it_wrong( __FUNCTION__, __( 'You are not allowed to clone this class.', 'order-manager-system' ), '1.0.0' );
		}

		/**
		 * Disable unserializing of the class.
		 *
		 * @access	public
		 * @since	1.0.0
		 * @return	void
		 */
		public function __wakeup() {
			_doing_it_wrong( __FUNCTION__, __( 'You are not allowed to unserialize this class.', 'order-manager-system' ), '1.0.0' );
		}

		/**
		 * Main Order_Manager_System Instance.
		 *
		 * Insures that only one instance of Order_Manager_System exists in memory at any one
		 * time. Also prevents needing to define globals all over the place.
		 *
		 * @access		public
		 * @since		1.0.0
		 * @static
		 * @return		object|Order_Manager_System	The one true Order_Manager_System
		 */
		public static function instance() {
			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Order_Manager_System ) ) {
				self::$instance					= new Order_Manager_System;
				self::$instance->base_hooks();
				self::$instance->includes();
				self::$instance->helpers		= new Order_Manager_System_Helpers();
				self::$instance->settings		= new Order_Manager_System_Settings();

				//Fire the plugin logic
				new Order_Manager_System_Run();

				/**
				 * Fire a custom action to allow dependencies
				 * after the successful plugin setup
				 */
				do_action( 'OMS/plugin_loaded' );
			}

			return self::$instance;
		}

		/**
		 * Include required files.
		 *
		 * @access  private
		 * @since   1.0.0
		 * @return  void
		 */
		private function includes() {
			require_once OMS_PLUGIN_DIR . 'core/includes/classes/class-order-manager-system-helpers.php';
			require_once OMS_PLUGIN_DIR . 'core/includes/classes/class-order-manager-system-settings.php';

			require_once OMS_PLUGIN_DIR . 'core/includes/classes/class-order-manager-system-run.php';
		}

		/**
		 * Add base hooks for the core functionality
		 *
		 * @access  private
		 * @since   1.0.0
		 * @return  void
		 */
		private function base_hooks() {
			add_action( 'plugins_loaded', array( self::$instance, 'load_textdomain' ) );
		}

		/**
		 * Loads the plugin language files.
		 *
		 * @access  public
		 * @since   1.0.0
		 * @return  void
		 */
		public function load_textdomain() {
			load_plugin_textdomain( 'order-manager-system', FALSE, dirname( plugin_basename( OMS_PLUGIN_FILE ) ) . '/languages/' );
		}

	}

endif; // End if class_exists check.